/**
 * Object that represents a timeline entry
 * @author: Daniel Puscher
 */
function TimelineEntry(el) {
    var $ = jQuery;

    var self = this;

    var privacy = PRIVACY.UNKNOWN;
    var privacyRead = false;
    var icon = "";
    var type = -1;
    var typeRead = false;

    // check if the element has an id, if not add a custom one
    if (!$(el).attr("id")) {
        $(el).attr("id", "tl_unit_" + randomString(10) + "_c");
    }
    this.id = $(el).attr("id");

    this.colorized = false;
    this.actionAdded = false;

    var element = el;

    var pr = privacyReader.getInstance();
    var pv = privacyVisualizer.getInstance();


    /**
     * This initializes the timeline entry object:
     * Read privacy and type
     */
    function init(){
        readPrivacy();
        readType();
    }

    /**
     * Will be thrown when an error occurs
     * @param message
     */
    function TimelineEntryException(message) {
        this.message = message;
        this.name = "TimelineEntryException";
        this.toString = function(){
            return this.name + ": " + this.message;
        }
    }

    /**
     * Returns the privacy
     * @param callback
     */
    this.getPrivacy = function(callback) {
        if (!privacyRead) {
            readPrivacy(callback);
        } else {
            callback(privacy);
        }
    };

    /**
     * Refresh the element
     */
    this.refresh = function() {
        privacyRead = false;
        self.colorized = false;

        readPrivacy(function(){
            self.colorize();
        });
    };

    /**
     * Read privacy settings of the element
     * @param {Function}    [callback]    Callback function, privacy value will be passed as attribute
     */
    function readPrivacy(callback) {
        pr.read(element, function (privacy_value, icon_value) {
            // save the read privacy setting and icon (base64 string)
            privacy = privacy_value;
            icon = icon_value;

            privacyRead = true;

            if(typeof callback === "function"){
                callback(privacy_value);
            }
        });
    }

    /**
     * Reads the type of the entry from its html-code, async
     * @param callback
     */
    function readType(callback) {
        var entryType = ENTRYTYPES.UNKNOWN;


        //Read feedback parameters from comment-field:
        var feedback_params_obj = {};

        // Check if it is possible to read the feedback parameters from the post
        // (it contains information about the type of the Entry)
        var feedback_params = $(element).find("[name=feedback_params]");

        // Does the element exist and does it have at least the value-attribute?
        if (feedback_params.length > 0 && $(feedback_params).is("[value]")) {
            feedback_params_obj = JSON.parse(feedback_params.first().attr("value"));
        }


        if($(element).find("div.audienceSelector").length > 0){
            entryType = ENTRYTYPES.CHANGEABLE;
        } else if($(element).find("span.fwb.fcg > i.mhs.img").length > 0){
            entryType = ENTRYTYPES.WALLPOST;
        } else if("actor" in feedback_params_obj &&
            runtime.userid != feedback_params_obj["actor"]){
            entryType = ENTRYTYPES.TAGGED;
        } else if($(element).find("a.fbPrivacyAudienceIndicator").length > 0){
            entryType = ENTRYTYPES.NOCHANGE;
        }

        if(typeof callback === "function"){
            callback(entryType);
        } else {
            return entryType;
        }
    }

    /**
     * Get the type of the timeline entry (integer value), async
     * @param callback
     */
    this.getType = function(callback) {
        if (!typeRead) {
            readType(callback);
        } else {
            callback(type);
        }
    };

    /**
     * Colorize the timeline entry
     */
    this.colorize = function() {
        if (!self.colorized) {
            // Read type and privacy of timeline-element
            self.getPrivacy(function (thePrivacy) {
                // Was the privacy read correctly?
                // + Check if the element is still not colorized, since this
                // method might be called multiple times concurrent
                if (thePrivacy != PRIVACY.UNKNOWN && !self.colorized) {
                    //Set colorized to true as fast as possible so the colorizing-method is not called more than once
                    self.colorized = true;
                    pv.colorize(element, thePrivacy);

                    // Set position of element to "relative" to ensure, absolute positioned elements
                    // are correctly placed inside element. (like the correct color-button)
                    $(element).css("position", "relative");
                }
            });
        }
    };

    /**
     * Add action to the element (view change- or message-overlay)
     */
    this.addAction = function(){
        if(!self.actionAdded){
            self.actionAdded = true;

            self.getType(function(el_type){
                self.getPrivacy(function(el_privacy){
                    if (el_type === ENTRYTYPES.CHANGEABLE) {
                        pv.addPrivacyChangeOverlay(element, pageTypes.TIMELINE, self.id, el_privacy, function(success){
                            if(success){
                                self.refresh();
                            }
                        });
                    } else {
                        pv.addMessageOverlay(element, el_type);
                    }
                });
            });
        }
    };

    /**
     * TODO: Delete this, only used for debugging!
     * @returns {{privacy: (PRIVACY.UNKNOWN|*), icon: string, type: number}}
     */
    this.getInfo = function() {
        return {
            id: this.id,
            element: element,
            privacy: privacy,
            icon: icon,
            type: type
        }
    };

    /**
     * Reset the element
     */
    this.reset = function(){
        privacy = PRIVACY.UNKNOWN;
        privacyRead = false;
        icon = "";
        type = -1;
        typeRead = false;

        pv.resetElement(element);
        this.colorized = false;
        this.actionAdded = false;
    };

    /**
     * Check if the element is still active in DOM
     * @returns {boolean}
     */
    this.active = function(){
        return $(element).closest('body').length > 0;
    };

    /**
     * Check if the element is in the viewport
     * @returns {boolean}
     */
    this.inView = function(){
        return $(element).isOnScreen();
    };

    // init the instance of the object!
    init();
}