/**
 * Image extractor cuts out an region from an image
 */
var imageExtractor = (function () {
    //Stores a reference to the instance (singleton-pattern)
    var instance;

    /**
     * Creates an instance of the image extractor
     * @returns {{extract: extract}}
     */
    function init() {
        var backgroundImageURLs = {};

        function extract(url, size, pos, callback){
            jQuery.when(saveBase64OfImage(url)).then(function () {
                cutOutRegion(url, size, pos, callback);
            });
        }

        /**
         * Reads the privacy by sub-image (defined by url of image, size and position), result is passed to callback function
         * @param {String} url          Image-URL
         * @param {Array} size          Size of sub-image [width, height]
         * @param {Array} pos           Position of the sub-image in image [x, y]
         * @param {Function} callback   Callback-function (
         */
        function cutOutRegion(url, size, pos, callback) {
            if (typeof backgroundImageURLs[url] !== "undefined") {
                var base64_img = "data:image/png;base64," + backgroundImageURLs[url];
                var tmp_img = document.createElement('img');
                tmp_img.onload = function () {
                    var canvas = document.createElement("canvas");
                    canvas.width = size[0];
                    canvas.height = size[1];

                    canvas.getContext('2d').drawImage(this, pos[0], pos[1], size[0], size[1], 0, 0, size[0], size[1]);
                    var b64 = canvas.toDataURL().split(",")[1]; //Get Base64-representation of partial image
                    callback(b64);
                };
                tmp_img.setAttribute("src", base64_img);
            }
            else {
                throw new ReaderException("Sorry, did not find base64 representation of image in cache. There may be an error downloading it.");
            }
        }


        function saveBase64OfImage(url) {
            var dfd = jQuery.Deferred();

            if (backgroundImageURLs[url] !== undefined) {
                dfd.resolve();
            }
            else {
                var req = new XMLHttpRequest();
                req.overrideMimeType('text/plain; charset=x-user-defined');
                req.open('GET', url, true);
                req.responseType = 'arraybuffer';
                req.onload = function () {
                    var buffer = req.mozResponseArrayBuffer || req.response;
                    var type = req.getResponseHeader('Content-Type');

                    var blob = new Blob([new Uint8Array(buffer)], {type: type});
                    var fr = new FileReader();
                    fr.onload = function () {
                        backgroundImageURLs[url] = window.btoa(fr.result);
                        dfd.resolve();
                    };
                    fr.readAsBinaryString(blob);
                };
                req.send(null);
            }

            return dfd.promise();
        }

        return {
            extract : extract
        };
    }

    return {
        // Get the Singleton instance if one exists
        // or create one if it doesn't
        getInstance: function () {
            if (!instance) {
                instance = init();
            }
            return instance;
        }
    };
})();