/**
 * Newsfeed Processor
 * Part of Facebook Privacy Watcher
 *
 * @author Daniel Puscher
 */

var newsfeedProcessor = (function ($) {

    /**
     * Stores the newsfeed entries that were processed in the current session
     * @type {{NewsfeedEntry}}
     */
    var cache = {};


    /**
     * Returns all newsfeed entries that are (partly) visible in the current view
     * @returns {Array}
     */
    function getInView() {
        var inView = [];

        _.each(cache, function (nfEntry, k) {
            if (nfEntry.inView()) {
                inView.push(nfEntry);
            }
        });

        return(inView);
    }

    /**
     * Resets all newsfeed entries and removes all colors
     */
    function resetAll() {
        // Reset the cache, otherwise later the refresh will not work correctly... :/
        privacyReader.getInstance().resetCache();

        _.each(cache, function (nf, k) {
            nf.reset();
        });
    }

    /**
     * Refreshes all newsfeed entries (re-read privacy and colorize again)
     */
    function refreshAll() {
        // Reset the cache, otherwise the refresh will not work correctly... :/
        privacyReader.getInstance().resetCache();

        _.each(cache, function (nf, k) {
            nf.refresh();
        });
    }

    /**
     * Process a single newsfeed entry and colorize it if it is in the current view
     * @param {HTMLElement} el  The timeline entry html element
     */
    function processSingleEntry(el) {
        if (!el.id || !cache[el.id]) {
            //No id present or not in list of newsfeed entries
            var nf_entry = new NewsfeedEntry(el);
            cache[nf_entry.id] = nf_entry;
        } else {
            var nf_entry = cache[el.id];
        }

        if (nf_entry.inView()) {
            // Colorize Entry:
            nf_entry.colorize();
        }
    }

    /**
     * Starts the processing of all entries found on the current page
     */
    function go() {
        if ($(pageIndicators.newsfeed).length > 0) {
            // Process new entries:
            $(".userContentWrapper:visible").parent().filter(":not([fpw])").each(function (k, v) {
                processSingleEntry(v);
            });

            // Process already processed entries that are in view now:
            _.each(getInView(), function (el) {
                el.colorize();
            });
        }
    }

    return {
        go: go,
        getInView: getInView,
        resetAll: resetAll,
        refreshAll: refreshAll
    };

})(jQuery);