function calculateBorderColor(bgColor) {
    function dec2hex(v) {
        return v < 0x10
            ? "0" + Math.max(0, v).toString(16)
            : Math.min(255, v).toString(16);
    }

    function hexColor2rgb(hexColor) {
        var r = 0, g = 0, b = 0;
        if (hexColor.length === 4) {
            r = hexColor.charAt(1);
            r += r;
            g = hexColor.charAt(2);
            g += g;
            b = hexColor.charAt(3);
            b += b;
        } else if (hexColor.length === 7) {
            r = hexColor.substring(1, 3);
            g = hexColor.substring(3, 5);
            b = hexColor.substring(5, 7);
        }
        return {
            r: parseInt(r, 16),
            g: parseInt(g, 16),
            b: parseInt(b, 16)
        };
    }

    function rgb2hsl(r, g, b) {
        var min = Math.min(r /= 255, g /= 255, b /= 255),
            max = Math.max(r, g, b),
            d = max - min,
            h,
            s,
            l = (max + min) / 2;
        if (d) {
            s = l < .5 ? d / (max + min) : d / (2 - max - min);
            if (r == max) h = (g - b) / d + (g < b ? 6 : 0);
            else if (g == max) h = (b - r) / d + 2;
            else h = (r - g) / d + 4;
            h *= 60;
        } else {
            s = h = 0;
        }

        return {
            h: h,
            s: s,
            l: l
        };
    }

    function hsl2rgb(h, s, l) {
        var m1,
            m2;

        // Some simple corrections for h, s and l.
        h = h % 360;
        if (h < 0) h += 360;
        s = s < 0 ? 0 : s > 1 ? 1 : s;
        l = l < 0 ? 0 : l > 1 ? 1 : l;

        // From FvD 13.37, CSS Color Module Level 3
        m2 = l <= .5 ? l * (1 + s) : l + s - l * s;
        m1 = 2 * l - m2;

        function v(h) {
            if (h > 360) h -= 360;
            else if (h < 0) h += 360;
            if (h < 60) return m1 + (m2 - m1) * h / 60;
            if (h < 180) return m2;
            if (h < 240) return m1 + (m2 - m1) * (240 - h) / 60;
            return m1;
        }

        function vv(h) {
            return Math.round(v(h) * 255);
        }

        return {
            r: vv(h + 120),
            g: vv(h),
            b: vv(h - 120)
        };
    }


    var rgbColor = hexColor2rgb(bgColor);
    var hslColor = rgb2hsl(rgbColor.r, rgbColor.g, rgbColor.b);
    hslColor.l = (hslColor.l - 0.14 > 1) ? 1 : (hslColor.l - 0.14);

    var newRgbColor = hsl2rgb(hslColor.h, hslColor.s, hslColor.l);

    return ("#" + dec2hex(newRgbColor.r) + dec2hex(newRgbColor.g) + dec2hex(newRgbColor.b));
}

function readCurrentSettings(reload) {
    getPref("background", function (currentColors) {
        getPref("animate", function (animate) {
            getPref("useImages", function (useImages) {
                if (animate === true) {
                    jQuery("#animate").attr('checked', '');
                    if (reload === true) {
                        jQuery("#animate").parent().find("a").addClass("checked");
                    }
                }
                else {
                    jQuery("#animate").removeAttr('checked');
                    if (reload === true) {
                        jQuery("#animate").parent().find("a").removeClass("checked");
                    }
                }

                var control = 'brightness';

                if (reload !== true) {
                    $('#colorPublic').minicolors({control: control, defaultValue: currentColors[0], change: function (hex) {
                        saveColors(hex, 0);
                    }});
                    $('#colorFriends').minicolors({control: control, defaultValue: currentColors[1], change: function (hex) {
                        saveColors(hex, 1);
                    }});
                    $('#colorOnlyMe').minicolors({control: control, defaultValue: currentColors[2], change: function (hex) {
                        saveColors(hex, 2);
                    }});
                    $('#colorCustom').minicolors({control: control, defaultValue: currentColors[3], change: function (hex) {
                        saveColors(hex, 3);
                    }});
                } else {
                    $('#colorPublic').minicolors("value", currentColors[0]);
                    $('#colorFriends').minicolors("value", currentColors[1]);
                    $('#colorOnlyMe').minicolors("value", currentColors[2]);
                    $('#colorCustom').minicolors("value", currentColors[3]);
                }

                if (useImages === true) {
                    //Disable Color inputs
                    setStatusOfColorInput(true);

                    jQuery("#useImages").attr('checked', '');
                    if (reload === true) {
                        jQuery("#useImages").parent().find("a").addClass("checked");
                    }
                }
                else {
                    //Enable Color inputs
                    setStatusOfColorInput(false);

                    jQuery("#useImages").removeAttr('checked');
                    if (reload === true) {
                        jQuery("#useImages").parent().find("a").removeClass("checked");
                    }
                }
            });
        });
    });
}

function setStatusOfColorInput(v) {
    $('#colorPublic').prop('disabled', v);
    $('#colorFriends').prop('disabled', v);
    $('#colorOnlyMe').prop('disabled', v);
    $('#colorCustom').prop('disabled', v);
    var buttonStatus = (v) ? "disable" : "enable";
    $('#colorPreset1').button(buttonStatus);
    $('#colorPreset2').button(buttonStatus);
}

function resetSettings() {
    resetPrefs();
    readCurrentSettings(true);
}

function clearCache() {
    setPref("cache", {});
}

function saveColors(val, i) {
    setPref("background", val, i);
    setPref("border", calculateBorderColor(val), i);
}

function showModal(title, content, buttonCaption, buttonAction) {
    var buttons = {};
    buttons[buttonCaption] = function () {
        buttonAction();
        $(this).dialog("close");
    };
    buttons[getLocString("cancel")] = function () {
        $(this).dialog("close");
    };

    $('<div />')
        .html(sprintf("<p><span class=\"ui-icon ui-icon-alert\" style=\"float: left; margin: 0 7px 20px 0;\"></span>%s</p>", content))
        .appendTo("body")
        .dialog({
            title: title,
            modal: true,
            width: 400,
            hide: "fade",
            show: "fade",
            buttons: buttons,
            resizable: false
        });
}

function notification(text) {
    noty({text: text,
        dismissQueue: false,
        timeout: 4000,
        closeWith: ['button', 'click'],
        type: 'success'
    });
}

function initOptions() {
    var defaultColors = ["#baffb2", "#ffdab2", "#ffa3a3", "#ccdaff"];

    //----- jQuery UI --------
    $("#resetPrefs").button().click(function (e) {
        showModal(getLocString("resetPrefs"), getLocString("resetPrefsAsk"), getLocString("resetPrefsButton"), function () {
            resetSettings();
            notification(getLocString('resetPrefsSuccess'));
        });
        e.preventDefault();
    });

    $("#clearCache").button().click(function (e) {
        showModal(getLocString("clearCache"), getLocString("clearCacheAsk"), getLocString("clearCache"), function () {
            clearCache();
            notification(getLocString('clearCacheSuccess'));
        });
        e.preventDefault();
    });

    $("#colorPreset1").button().click(function (e) {
        saveColors(defaultColors[0], 0);
        saveColors(defaultColors[1], 1);
        saveColors(defaultColors[2], 2);
        saveColors(defaultColors[3], 3);
        readCurrentSettings(true);
        e.preventDefault();
    });

    $("#colorPreset2").button().click(function (e) {
        saveColors(defaultColors[2], 0);
        saveColors(defaultColors[1], 1);
        saveColors(defaultColors[0], 2);
        saveColors(defaultColors[3], 3);
        readCurrentSettings(true);
        e.preventDefault();
    });

    //-------- PrettyCheckable ------------
    jQuery("#animate").on('change', function () {
        setPref("animate", $(this).prop("checked"));
    });

    jQuery("#useImages").on('change', function () {
        setPref("useImages", $(this).prop("checked"));
        setStatusOfColorInput($(this).prop("checked"));
    });

    readCurrentSettings();

    jQuery("#animate").prettyCheckable();
    jQuery("#useImages").prettyCheckable();

};

var locStrings = {};

self.port.on("l10nStrings", function (s) {
    locStrings = s;
    loadI18nStrings();
});

function getLocString(n) {
    if (locStrings[n]) {
        return locStrings[n];
    }
    return "(Unknown String)";
}

self.port.on("storageObject", function (storage) {
    tmpStorage = storage;
    initOptions();
});