/**
 * PrivacyVisualizer lets you colorize html elements depending on their privacy setting
 * @constructor
 * @param [new_settings]
 * @author Daniel Puscher
 */
var privacyVisualizer = (function ($) {
    //Stores a reference to the instance (singleton-pattern)
    var instance;

    /**
     * Creates an instance of the privacy visualizer
     * @returns {{updateSettings: updateSettings, colorize: colorize, resetElement: resetElement, addPrivacyChangeOverlay: addPrivacyChangeOverlay, addMessageOverlay: addMessageOverlay}}
     */
    function init() {
        /**
         * Initial settings, may be changed by parameter or later using function
         */
        var background = ["#BAFFB2", "#FFDAB2", "#FFA3A3", "#CCDAFF"];
        var backgroundImages = _.map(["img/bg_noColor_0.gif", "img/bg_noColor_1.gif", "img/bg_noColor_2.gif", "img/bg_noColor_3.gif"], getFileUrl);
        var border = ["#7DFF6E", "#FFB25E", "#FF6363", "#8FAEFF"];
        var borderImages = ["#333333", "#333333", "#333333", "#333333"];
        var animate = true;
        var useImages = false;

        var pc = privacyChanger.getInstance();

        // Links that are shown in message overlays and tell the user where he can change the privacy settings
        var changeLinks = {
            wallpost: "http://www.facebook.com/settings?tab=timeline&section=others",
            tagged: "http://www.facebook.com/settings?tab=timeline&section=tagging",
            friendlist: "http://www.facebook.com/profile.php?sk=friends",
            about: "http://www.facebook.com/profile.php?sk=info"
        };

        /**
         * Colorize the background of the element
         */
        function colorizeBackground(el, setting) {
            if (useImages === true) {
                el.style.backgroundImage = sprintf("url(%s)", backgroundImages[setting]);
            }
            else {
                if (animate === true) {
                    $(el).animate({backgroundColor: background[setting]});
                }
                else {
                    el.style.backgroundColor = background[setting];
                }
            }
        }

        /**
         * Colorize the border of the element
         */
        function colorizeBorder(el, setting) {
            if (el.style.border === "") {
                //Only set a 1px transparent border if none is set yet (to make the animation work)
                el.style.border = "1px solid transparent";
            }

            var bo = (!!useImages) ? borderImages : border;

            if (animate) {
                $(el).animate({borderColor: bo[setting]});
            }
            else {
                el.style.borderColor = bo[setting];
            }
        }

        /**
         * Colorize the text of the element
         * @param el
         * @param setting
         */
        function colorizeText(el, setting) {
            if (setting >= 0 && setting <= 3) {
                $(el).css({
                    "color": background[setting],
                    "text-shadow": "-1px 0 " + border[setting] + ", 0 1px " + border[setting] + ", 1px 0 " + border[setting] + ", 0 -1px " + border[setting]
                });
            }
        }

        function setColorCssProperty(el, setting, property) {
            var borderImg = (!!useImages) ? borderImages[setting] : border[setting];

            if (!!animate) {
                var animate_obj = {};
                animate_obj[property] = borderImg;
                $(el).animate(animate_obj);
            }
            else {
                $(el).css(property, borderImg);
            }
        }

        function resetColorCssProperty(el, property) {
            var change_css = {};
            change_css[property] = "";

            if (!!animate) {
                $(el).animate(change_css, function () {
                    $(el).css(change_css);
                });
            }
            else {
                $(el).css(change_css);
            }
        }

        /**
         * Colorize the given element depending on its privacy setting and it depending on its type
         *
         * @param setting   Privacy setting of the element {0, 1, 2, 3)
         * @param el        Element to be colorized
         * @param [type]    Type of the element (optional)
         */
        function colorize(el, setting, type) {
            if (setting >= 0 && setting <= 3) {
                if (type === ENTRYTYPES.FRIENDLIST || type === ENTRYTYPES.LIKES) {
                    colorizeBackground(el, setting);
                    $(el).find("img").animate({opacity: .6}).addClass("fpwImg");
                }
                else if (type === "noborder") {
                    colorizeBackground(el, setting);
                }
                else {
                    colorizeBackground(el, setting);
                    colorizeBorder(el, setting);
                }

                $(el).attr("fpw", "colored").attr("fpwprivacy", setting);

                /**
                 * TODO: Add the ribbon which allows the users to correct the colors
                 */
                /*var ribbon = $("<div/>");
                 ribbon.addClass("fpw_ribbon");
                 ribbon.html('<img src="' + getFileUrl("img/color_ribbons.png") + '">');

                 $(el).append(ribbon);

                 if(animate){
                 $(ribbon).animate({opacity: 1});
                 } else {
                 $(ribbon).css("opacity", 1);
                 }*/
            }
            else {
                console.log(["[Error] Wrong privacy setting. Must be in range 0...3.", el, setting, type]);
            }
        }

        function copyTooltipToElement(el) {
            var subEl = $(el).find("a.uiSelectorButton[aria-label], a.fbPrivacyAudienceIndicator[aria-label]")[0];
            if (subEl && $(subEl).attr("aria-label")) {
                $(el).attr("data-hover", "tooltip");
                $(el).attr("aria-label", sprintf("%s: %s", getLocString("visibleFor"), $(subEl).attr("aria-label")));
                $(el).attr("data-tooltip-alignh", "center");
            }
        }

        /**
         * Show the message overlay
         * @param {Event}   event   The click event
         * @param {String}  type    Type of the entry
         * @returns {boolean}
         */
        function showMessageOverlay(event, type) {
            removePrivacyChangeOverlay();

            var target = (event.toElement || event.target);
            var nodename = target.nodeName;
            while (nodename != "DIV") {
                if (nodename == "IMG") return false;
                if (nodename == "INPUT") return false;
                if (nodename == "TEXTAREA") return false;
                if (nodename == "A") return false;
                if (nodename == "BODY") break;

                target = target.parentNode;
                nodename = target.nodeName;
            }

            var div = generateMessagePopup(type);
            $(div).css("top", event.pageY).css("left", event.pageX);
            addElement(document.body, div);

            return true;
        }

        /**
         * Generate a message popup for a certain entry type
         * @param {String}  type    Type of the entry
         * @returns {HTMLElement}
         */
        function generateMessagePopup(type) {
            var div = document.createElement("div");
            div.id = "fpwChangeOverlay";
            div.className = "message";
            div.onclick = function (e) {
                removePrivacyChangeOverlay();
                e.stopPropagation();
            };

            var pattern = "%s<br><br><a href=\"%s\">%s</a>";
            var messageText = getLocString("msg_unknown");


            if (type == ENTRYTYPES.WALLPOST) {
                messageText = sprintf(pattern, getLocString("msg_wallpost"), changeLinks.wallpost, getLocString("changePrivacy"));
            }
            else if (type == ENTRYTYPES.TAGGED) {
                messageText = sprintf(pattern, getLocString("msg_tagged"), changeLinks.tagged, getLocString("changePrivacy"));
            }
            else if (type == ENTRYTYPES.ABOUT) {
                messageText = sprintf(pattern, getLocString("msg_about"), changeLinks.about, getLocString("changePrivacy"));
            }
            else if (type == ENTRYTYPES.NOCHANGE) {
                messageText = getLocString("msg_nochange");
            }
            /*else if (type == ENTRYTYPES.FRIENDLIST) {
             messageText = sprintf(pattern, getLocString("msg_friendlist"), changeLinks.friendlist, getLocString("changePrivacy"));
             }
             else if (type == ENTRYTYPES.ACTIVITYNOCHANGE) {
             messageText = getLocString("msg_activitynochange");
             }
             else if (type == ENTRYTYPES.ALBUMSNOCHANGE) {
             messageText = getLocString("msg_albumsnochange");
             }
             else if (type == ENTRYTYPES.LIKES) {
             messageText = getLocString("msg_likes");
             }*/

            div.innerHTML = messageText;

            return div;
        }

        /**
         * Show a message overlay when the privacy can not be changed directly
         * @param {HTMLElement} el  The Element to which the action is added
         * @param {String}  type    Type of the message to display
         */
        function addMessageOverlay(el, type) {
            var f = function (e) {
                if (showMessageOverlay(e, type)) {
                    e.stopPropagation();
                }
            };

            el.onclick = f;
            el.oncontextmenu = f;
        }

        /**
         * Add the privacy change overlay to an element
         * @param {HTMLElement} el  The Element to which the action is added
         * @param {String}  page    Type of the page that is displayed
         * @param {String}  id      ID of the element that needs to be changed
         * @param {Number}  oldVal  Old privacy setting
         * @param {Function}    callback    Callback function
         */
        function addPrivacyChangeOverlay(el, page, id, oldVal, callback) {
            el.onclick = function (e) {
                if (showPrivacyChangeOverlay(e, page, id, oldVal, callback)) {
                    e.stopPropagation();
                }
            };
            el.oncontextmenu = el.onclick;
        }

        /**
         * Show the change overlay
         * @param {Event}   event   The click event
         * @param {String}  page    Type of the page that is displayed
         * @param {String}  id      ID of the element that needs to be changed
         * @param {Number}  oldVal  Old privacy setting
         * @param {Function}    callback    Callback function
         * @returns {boolean}
         */
        function showPrivacyChangeOverlay(event, page, id, oldVal, callback) {
            removePrivacyChangeOverlay();

            var target = (event.toElement || event.target);
            var nodename = target.nodeName;
            while (nodename != "DIV") {
                if (nodename == "IMG") return false;
                if (nodename == "INPUT") return false;
                if (nodename == "TEXTAREA") return false;
                if (nodename == "A") return false;
                if (nodename == "BUTTON") return false;
                if (nodename == "BODY") break;

                target = target.parentNode;
                nodename = target.nodeName;
            }

            if ($(target).hasClass("uiInlineTokenizer")) return false;
            if ($(target).hasClass("tokenarea")) return false;

            var div = generatePrivacyChangeOverlay(page, id, oldVal, callback);
            $(div).css("top", event.pageY).css("left", event.pageX);
            addElement(document.body, div);

            return true;
        }

        /**
         * Generate the html element to change the privacy
         * @param {String}  page    Type of the page that is displayed
         * @param {String}  id      ID of the element that needs to be changed
         * @param {Number}  oldVal  Old privacy setting
         * @param {Function}    callback    Callback function
         * @returns {HTMLElement}
         */
        function generatePrivacyChangeOverlay(page, id, oldVal, callback) {
            var div = document.createElement('div');
            div.id = "fpwChangeOverlay";

            var can = document.createElement('canvas');
            $(can).attr("width", 140).attr("height", 140).css("marginTop", "-70px").css("marginLeft", "-70px");

            var ctx = can.getContext('2d');

            var canvasBackground = (useImages === true) ? backgroundImages : background;
            var canvasBorder = (useImages === true) ? borderImages : border;
            generatePrivacyOverlayButton(ctx, Math.PI * 1.5, 0, canvasBackground[0], canvasBorder[0], 4, -4, useImages);         //top right
            generatePrivacyOverlayButton(ctx, 0, Math.PI / 2, canvasBackground[1], canvasBorder[1], 4, 4, useImages);            //bottom right
            generatePrivacyOverlayButton(ctx, Math.PI / 2, Math.PI, canvasBackground[2], canvasBorder[2], -4, 4, useImages);     //bottom left
            generatePrivacyOverlayButton(ctx, Math.PI, Math.PI * 1.5, canvasBackground[3], canvasBorder[3], -4, -4, useImages);  //top left

            ctx.globalCompositeOperation = "destination-over";
            ctx.beginPath();
            ctx.fillStyle = 'rgba(255,255,255,.5)';
            ctx.moveTo(70, 70);
            ctx.arc(70, 70, 56.5, 0, Math.PI * 2, true);
            ctx.closePath();
            ctx.shadowBlur = 20;
            ctx.shadowColor = "black";
            ctx.fill();

            $(can).click(function (e) {
                var x = Math.floor((e.pageX - $(can).offset().left) / 70);
                var y = Math.floor((e.pageY - $(can).offset().top) / 70);
                var newSetting = (x === 0) ? ((y === 0) ? 3 : 2) : ((y === 0) ? 0 : 1);
                pc.change(page, id, newSetting, oldVal, callback);
                removePrivacyChangeOverlay();
                e.stopPropagation();
            });

            div.appendChild(can);

            var img = document.createElement('img');
            img.src = getFileUrl("img/overlay_close.png");
            img.onclick = function (e) {
                removePrivacyChangeOverlay();
                e.stopPropagation();
            };

            div.appendChild(img);

            return div;
        }

        /**
         * Remove the overlay that is used to change the privacy
         */
        function removePrivacyChangeOverlay() {
            removeElementsById("fpwChangeOverlay");
        }

        /**
         * Draws an overlay button to the given context
         * @param {object}  ctx   Context where the button gets drawn in
         * @param {number}  start Start-degree of the button [0 ... 2*PI]
         * @param {number}  end   End-degree of the button [0 ... 2*PI]
         * @param {string}  background    Background-color
         * @param {string}  border    Border-color
         * @param {number}  [offsetX]   X-offset
         * @param {number}  [offsetY]   Y-offset
         * @param {boolean} [image] true if the given background value is a image, false otherwise
         */
        function generatePrivacyOverlayButton(ctx, start, end, background, border, offsetX, offsetY, image) {
            offsetX = (offsetX) ? offsetX : 0;
            offsetY = (offsetY) ? offsetY : 0;

            var orig_x = 70;
            var orig_y = 70;

            var x = orig_x + offsetX;
            var y = orig_y + offsetY;

            var canvasBackground = background;
            if (image === true) {
                var objImg = document.createElement('img');
                objImg.src = background;
                canvasBackground = ctx.createPattern(objImg, "repeat");
            }

            ctx.globalCompositeOperation = "destination-over";
            ctx.beginPath();
            ctx.arc(x, y, 50, start, end, false);
            ctx.lineTo(x, y);
            ctx.closePath();
            ctx.fillStyle = canvasBackground;
            ctx.fill();
            ctx.strokeStyle = border;
            ctx.lineWidth = 4;
            ctx.stroke();

            ctx.globalCompositeOperation = "destination-out";
            ctx.beginPath();
            ctx.lineTo(orig_x, orig_y);
            ctx.arc(orig_x, orig_y, 15, 0, Math.PI * 2, false);
            ctx.fill();
            ctx.closePath();
        }

        /**
         * Reset the element (remove event listeners and colors)
         * @param {HTMLElement}     el  The element to reset
         */
        function resetElement(el) {
            // Disable actions on click or context menu!
            el.onclick = null;
            el.oncontextmenu = null;

            if (animate === true) {
                $(el).animate({backgroundColor: "none", borderColor: "none", backgroundImage: "none"}, function () {
                    $(el).css("background", "")
                        .css("border", "")
                        .removeAttr("fpw")
                        .removeAttr("fpwprivacy");
                });

                $(el).find("img.fpwImg").animate({opacity: 1});
            }
            else {
                $(el).css("background", "")
                    .css("border", "")
                    .removeAttr("fpw")
                    .removeAttr("fpwprivacy");

                $(el).find("img.fpwImg").css("opacity", 1);
            }
        }

        /**
         * Change the default settings
         * @param s     New settings for the visualizer (background, border, animate, useImages)
         */
        function updateSettings(s) {
            if (s.hasOwnProperty("useImages")) {
                useImages = s["useImages"];
            }
            if (s.hasOwnProperty("background")) {
                background = s["background"];
            }
            if (s.hasOwnProperty("border")) {
                border = s["border"];
            }
            if (s.hasOwnProperty("animate")) {
                animate = s["animate"];
            }
        }

        function addElement(parent, child) {
            if (animate === true) {
                $(child).hide();
                $(parent).append(child);
                $(child).fadeIn();
            }
            else {
                parent.appendChild(child);
            }
        }

        /**
         * Define public functions
         */
        return {
            updateSettings: updateSettings,
            colorize: colorize,
            colorizeText: colorizeText,
            resetElement: resetElement,
            addPrivacyChangeOverlay: addPrivacyChangeOverlay,
            addMessageOverlay: addMessageOverlay,
            setColorCssProperty: setColorCssProperty,
            resetColorCssProperty: resetColorCssProperty
        };
    }

    return {
        // Get the Singleton instance if one exists
        // or create one if it doesn't
        getInstance: function () {
            if (!instance) {
                instance = init();
            }
            return instance;
        }
    };
})(jQuery);
