/**
 * Timeline Processor
 * Part of Facebook Privacy Watcher
 *
 * @author Daniel Puscher
 */

var timelineProcessor = (function ($) {

    /**
     * Stores the timeline entries that were processed in the current session
     * @type {{TimelineEntry}}
     */
    var cache = {};
    var composerEl = {
        input: null,
        arrow: null
    };

    /**
     * Returns all timeline entries that are (partly) visible in the current view
     * @returns {Array}
     */
    function getInView() {
        var en_in_view = [];

        _.each(cache, function (tl, k) {
            if (tl.inView()) {
                en_in_view.push(tl);
            }
        });

        return(en_in_view);
    }

    /**
     * Resets all timeline entries and removes all colors
     */
    function resetAll() {
        // Reset the cache, otherwise later the refresh will not work correctly... :/
        privacyReader.getInstance().resetCache();

        _.each(cache, function (tl, k) {
            tl.reset();
        });

        var pv = privacyVisualizer.getInstance();
        if (!!composerEl.input) {
            pv.resetElement(composerEl.input);
        }

        if (!!composerEl.arrow) {
            pv.resetColorCssProperty(composerEl.arrow, "borderBottomColor");
        }
    }

    /**
     * Refreshes all timeline entries (re-read privacy and color again)
     */
    function refreshAll() {
        // Reset the cache, otherwise the refresh will not work correctly... :/
        privacyReader.getInstance().resetCache();

        _.each(cache, function (tl, k) {
            tl.refresh();
        });

        processComposer();
    }

    /**
     * Process a single timeline entry and colorize it if it is in the current view
     * @param {HTMLElement} el  The timeline entry html element
     */
    function processSingleEntry(el) {
        $(el).attr("fpw", "processed");
        var el_id = $(el).attr("id");

        if (!el_id) {
            //Element does not have an id, so create a random one and assign it to the html element
            var new_id = "tl_unit_" + randomString(20) + "_c";
            el_id = new_id;
            $(el).attr("id", new_id);
        }

        var tl_entry = cache[el_id];

        if (!(tl_entry instanceof TimelineEntry)) {
            // Not processed, create new timeline-entry
            tl_entry = new TimelineEntry(el);

            cache[el_id] = tl_entry;
        }

        if (tl_entry.inView()) {
            // Colorize Entry:
            tl_entry.colorize();

            // Add action to entry. Show message or change-icon!
            tl_entry.addAction();
        }
    }

    function processComposer() {
        var pr = privacyReader.getInstance();
        var pv = privacyVisualizer.getInstance();

        var composer = $(".fbTimelineComposerUnit")[0];
        if (composer) {
            pr.read(composer, function (setting) {
                composerEl.input = $(composer).find(".uiMentionsInput")[0];
                composerEl.arrow = $(composer).find("span.uiIconText > i:not(.img)")[0];

                if (!!composerEl.input) {
                    pv.colorize(composerEl.input, setting);
                }

                if (!!composerEl.arrow) {
                    pv.setColorCssProperty(composerEl.arrow, setting, "borderBottomColor");
                }
            });
        }
    }

    /**
     * Starts the processing of all entries found on the current page
     */
    function go() {
        // Only process page if timeline is shown
        if ($(pageIndicators.timeline).length > 0) {
            // Process entries that were not processed yet:
            $(".userContentWrapper:visible").parent().filter(":not([fpw])").each(function (k, v) {
                processSingleEntry(v);
            });

            processComposer();

            // Process already processed entries that are in view now:
            _.each(getInView(), function (tlEntry) {
                tlEntry.colorize();
                tlEntry.addAction();
            });
        }
    }

    return {
        go: go,
        getInView: getInView,
        resetAll: resetAll,
        refreshAll: refreshAll
    };

})(jQuery);