/**
 * Load libraries
 */
var data = require("sdk/self").data;
var ss = require('sdk/simple-storage');
var tabs = require('sdk/tabs');
var pageMod = require("sdk/page-mod");
var ui = require("sdk/ui");

var { ActionButton } = require("sdk/ui/button/action");
var tab_urls = {};


/**
 * Define functions that are needed in the background-script
 */
function produceLocObject(names) {
    var locObj = {};
    for (var i = 0; i < names.length; i++) {
        locObj[names[i]] = require("sdk/l10n").get(names[i]);
    }
    return locObj;
}

/**
 * Generates a random string of the given length
 * @param {number}  [length]    Length of the random String (default: 20)
 * @returns {string}    A random string of given length
 */
function randomString(length) {
    length = (length || 10);

    var text = "";
    var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

    for (var i = 0; i < length; i++)
        text += possible.charAt(Math.floor(Math.random() * possible.length));

    return text;
}

/**
 * Return the addon version
 * @returns {String}
 */
function getAddonVersion() {
    return require("sdk/self").version;
}

/**
 * Get the storage element
 * @returns {storage|*}
 */
function getStorage() {
    return ss.storage;
}

/**
 * Get preference by name
 * @param {String}  name    Key of the preference
 * @returns {*}
 */
function getPref(name) {
    return ss.storage[name];
}

/**
 * Set the value of a preference
 * @param {String}  name    Key of the preference
 * @param {*}   value   Value of the preference
 */
function setPref(name, value) {
    ss.storage[name] = value;
}

/**
 * Return the url of a file inside the extension
 * @type {Function}
 */
var getFileUrl = data.url;

/**
 * Create a new tab
 * @type {Function}
 */
var createTab = tabs.open;

/**
 * Call function when a tab or its URL is updated
 * @param f
 */
function addTabsOnUpdatedListener(f) {
    tabs.on('ready', function (tab) {
        tab_urls[tab.id] = tab.url;
        f(tab.id, tab.url);
    });

    setInterval(function () {
        for each (var tab in tabs) {
            if (tab_urls[tab.id] != tab.url) {
                tab_urls[tab.id] = tab.url;
                f(tab.id, tab.url);
            }
        }
    }, 500);
}


/**
 * Panel that is shown when page action is clicked
 */
function openSettings() {
    tabs.open(data.url("options.html"));
    fpwPanel.hide();
}

function openFeedback() {
    tabs.open(data.url("feedback.html"));
    fpwPanel.hide();
}

function openAbout() {
    tabs.open(data.url("about.html"));
    fpwPanel.hide();
}

function openHelp() {
    ss.storage.showHelp = true;
    ss.storage.active = true;
    activeChanged(true);
    tabs.open("https://www.facebook.com/profile.php");
    fpwPanel.hide();
}

function activeChanged(state) {
    var icon = "img/icon-16.png";
    if (state !== true) {
        icon = "img/icon-16-sw.png"
    }
    widget.contentURL = data.url(icon);

    if (state !== true) {
        sendMessageToAllWorkers("clearPage");
    }
    else {
        sendMessageToAllWorkers("processPage");
    }
}

var fpwPanel = require("sdk/panel").Panel({
    width: 360,
    height: 175,
    contentURL: data.url("popup.html"),
    contentScriptWhen: "start",
    contentScriptFile: [data.url("lib/jquery-2.1.1.min.js"),
        data.url("lib/jquery-ui-1.9.2.min.js"),
        data.url("lib/prettyCheckable/prettyCheckable.js"),
        data.url("lib/sprintf.min.js"),
        data.url("popup.js"),
        data.url("pref.js")],
    onShow: function () {
        fpwPanel.port.emit("storageObject", getStorage());

        fpwPanel.port.on("open-settings-tab", openSettings);
        fpwPanel.port.on("open-feedback-tab", openFeedback);
        fpwPanel.port.on("open-about-tab", openAbout);
        fpwPanel.port.on("open-help", openHelp);
        fpwPanel.port.on("setPref", setPref);
        fpwPanel.port.on("activeChanged", activeChanged);
    },
    onHide: function () {
        fpwPanel.port.removeListener("open-settings-tab", openSettings);
        fpwPanel.port.removeListener("open-feedback-tab", openFeedback);
        fpwPanel.port.removeListener("open-about-tab", openAbout);
        fpwPanel.port.removeListener("open-help", openHelp);
        fpwPanel.port.removeListener("setPref", setPref);
        fpwPanel.port.removeListener("activeChanged", activeChanged);
    }
});

var widget = require("sdk/widget").Widget({
    id: "fpw",
    label: "Facebook Privacy Watcher",
    contentURL: data.url("img/icon-16.png"),
    panel: fpwPanel
});


/**
 * Options Page
 */
pageMod.PageMod({
    include: data.url("options.html"),
    contentScriptFile: [
        data.url("lib/jquery-2.1.1.min.js"),
        data.url("lib/jquery.minicolors.js"),
        data.url("lib/jquery-ui-1.9.2.min.js"),
        data.url("lib/prettyCheckable/prettyCheckable.js"),
        data.url("lib/noty/jquery.noty.js"),
        data.url("lib/noty/layouts/top.js"),
        data.url("lib/noty/themes/default.js"),
        data.url("lib/sprintf.min.js"),
        data.url("lib/loadI18nStrings.js"),
        data.url("pref.js"),
        data.url("options.js")],
    onAttach: function (worker) {
        var locStrings = ["options", "colors", "colorsDesc", "public", "friends", "colorPreset1", "onlyme", "custom", "colorPreset2", "animate", "useImages", "resetPrefsHeadline", "resetPrefs", "clearCache", "clearCacheDesc", "clearCache", "cancel", "resetPrefs", "resetPrefsAsk", "resetPrefsButton", "resetPrefsSuccess", "clearCache", "clearCacheAsk", "clearCacheSuccess"];
        worker.port.emit("l10nStrings", produceLocObject(locStrings));
        worker.port.emit("storageObject", getStorage());

        worker.port.on("setPref", function (name, value) {
            setPref(name, value);
        });
    }
});


/**
 * Feedback Page
 */
pageMod.PageMod({
    include: data.url("feedback.html"),
    contentScriptFile: [
        data.url("lib/jquery-2.1.1.min.js"),
        data.url("lib/jquery.sha1.js"),
        data.url("lib/jquery-ui-1.9.2.min.js"),
        data.url("lib/prettyCheckable/prettyCheckable.js"),
        data.url("lib/noty/jquery.noty.js"),
        data.url("lib/noty/layouts/top.js"),
        data.url("lib/noty/layouts/center.js"),
        data.url("lib/noty/themes/default.js"),

        data.url("lib/pidcrypt.js"),
        data.url("lib/pidcrypt_util.js"),
        data.url("lib/asn1.js"),
        data.url("lib/jsbn.js"),
        data.url("lib/rng.js"),
        data.url("lib/prng4.js"),
        data.url("lib/rsa.js"),
        data.url("lib/md5.js"),
        data.url("lib/aes_core.js"),
        data.url("lib/aes_cbc.js"),

        data.url("lib/sprintf.min.js"),
        data.url("lib/loadI18nStrings.js"),
        data.url("pref.js"),
        data.url("feedback.js")],
    onAttach: function (worker) {
        var locStrings = ["feedback", "feedbackIntro1", "feedbackIntro2", "veryGood", "good", "medium", "bad", "veryBad", "noOpinion", "greenPublic", "greenOnlyMe", "veryImportant", "important", "unimportant", "veryUnimportant", "feedbackQ1", "feedbackQ2", "feedbackQ3", "feedbackQ4", "feedbackQ5", "feedbackQ6", "sendFeedback", "feedbackSuccess", "feedbackError"];
        worker.port.emit("storageObject", getStorage());
        worker.port.emit("l10nStrings", produceLocObject(locStrings));
    }
});


/**
 * About Page
 */
pageMod.PageMod({
    include: data.url("about.html"),
    contentScriptFile: [
        data.url("lib/jquery-2.1.1.min.js"),
        data.url("lib/jquery-ui-1.9.2.min.js"),
        data.url("lib/sprintf.min.js"),
        data.url("lib/loadI18nStrings.js"),
        data.url("about.js")],
    onAttach: function (worker) {
        var locStrings = ["about", "aboutText", "aboutDeveloper", "aboutPress", "aboutLicense", "supportCased"];
        worker.port.emit("l10nStrings", produceLocObject(locStrings));
    }
});


/**
 * Save the workers:
 * @type {Array}
 */
var fpwWorkers = [];

/**
 * Send a message to all workers:
 * @param msg
 * @param param
 */
function sendMessageToAllWorkers(msg, param) {
    fpwWorkers.forEach(function (worker) {
        worker.port.emit(msg, param);
    });
}

/**
 * Remove a worker from the worker array
 * @param worker
 * @param workerArray
 */
function detachWorker(worker, workerArray) {
    var index = workerArray.indexOf(worker);
    if (index != -1) {
        workerArray.splice(index, 1);
    }
}

/**
 * And this is where the magic happens! :)
 * Append Javascript to Facebook
 */
var fpw = pageMod.PageMod({
    include: "*.facebook.com",
    contentScriptFile: [
        data.url("lib/jquery-2.1.1.min.js"),
        data.url("lib/jquery.color-2.1.0.min.js"),
        data.url("lib/jquery.addons.js"),
        data.url("lib/jquery.sha1.js"),
        data.url("lib/jquery.scrollto.js"),
        data.url("lib/jquery.scrollstop.js"),
        data.url("lib/jquery.isonscreen.js"),
        data.url("lib/underscore-min.js"),
        data.url("lib/sprintf.min.js"),

        data.url("helper.js"),
        data.url("pref.js"),

        data.url("TimelineEntry.js"),
        data.url("NewsfeedEntry.js"),
        data.url("imageExtractor.js"),
        data.url("privacyChanger.js"),
        data.url("privacyReader.js"),
        data.url("privacyVisualizer.js"),
        data.url("timelineProcessor.js"),
        data.url("newsfeedProcessor.js"),
        data.url("fpwHelp.js"),
        data.url("content.js"),
        data.url("fpw.js")],
    contentStyleFile: [data.url("fpw.css")],
    onAttach: function (worker) {
        worker.port.emit("resourceUrl", data.url());
        var locStrings = ["public", "friends", "onlyme", "custom", "msg_unknown", "msg_tagged", "msg_wallpost",
            "msg_nochange", "msg_friendlist", "msg_about", "msg_activitynochange", "msg_albumsnochange",
            "msg_likes", "changePrivacy", "visibleFor", "infoParsing", "infoParsing2", "firstStart1",
            "firstStart2", "firstStart3", "firstStart4", "firstStart5", "firstStart6", "helpNext", "helpText0",
            "helpText1", "helpText2", "helpText3", "helpText4", "helpText5", "helpTextEnd1", "helpTextEnd2",
            "createPublicEvent1", "createPublicEvent2", "createPublicEvent3", "newsfeed_like_public",
            "newsfeed_like_friends", "newsfeed_like_custom"];
        worker.port.emit("l10nStrings", produceLocObject(locStrings));
        worker.port.emit("storageObject", getStorage());

        worker.port.on("setPref", function (name, value) {
            setPref(name, value);
        });
        worker.port.on("open-feedback-tab", openFeedback);

        fpwWorkers.push(worker);
        worker.on('detach', function () {
            detachWorker(this, fpwWorkers);
        });
    }
});


/**
 * Initial values for preferences
 */
function initPrefs() {
    if (!getPref("active"))
        setPref("active", true);

    if (!getPref("background"))
        setPref("background", ["#baffb2", "#ffdab2", "#ffa3a3", "#ccdaff"]);

    if (!getPref("border"))
        setPref("border", ["#7aff6b", "#ffb86b", "#ff5c5c", "#85a6ff"]);

    if (!getPref("useImages"))
        setPref("useImages", false);

    if (!getPref("animate"))
        setPref("animate", true);

    if (!getPref("cache"))
        setPref("cache", {});

    if (!getPref("initial"))
        setPref("initial", {});

    if (!getPref("filled"))
        setPref("filled", {});

    if (!getPref("showHelp"))
        setPref("showHelp", true);

    if (!getPref("stats"))
        setPref("stats", {});

    if (!getPref("firstStart"))
        setPref("firstStart", new Date().getTime());

    if (!getPref("userBehaviorAsked"))
        setPref("userBehaviorAsked", false);

    if (!getPref("userBehaviorAnswer"))
        setPref("userBehaviorAnswer", -1);
}

initPrefs();
activeChanged(getPref("active"));


/**********************************************************************
 * Actions that are called when the addon is loaded (once per session)
 **********************************************************************/

// Show the "first start"-document when the addon is installed
// and run for the first time
if (!getPref("lastStartVersion")) {
    createTab(getFileUrl('docs/first_start.html'));
    setPref("lastStartVersion", getAddonVersion());
}
else if (getPref("lastStartVersion") !== getAddonVersion()) {
    //Update Detected
    setPref("lastStartVersion", getAddonVersion());

    // TODO: maybe show a changelog?
}

// Set an random installation ID
if (!getPref("installationID")) {
    setPref("installationID", randomString(30));
}